//---------------------------------------------------------------------------

#include <vcl.h>
#include "Clock.h"

#pragma hdrstop


#pragma package(smart_init)

//---------------------------------------------------------------------------
// Mit ValidCtrCheck wird sichergestellt, da die erzeugten Komponenten
// keine rein virtuellen Funktionen besitzen.
//
static inline void ValidCtrCheck(TClock *)
{
  new TClock(NULL);
}
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
__fastcall TClock::TClock(TComponent* Owner)
  : TCustomControl(Owner)
{
  nCompoSize =  800;
  nZeigerFaktor = 1;
  clBkUhrColor = clSkyBlue;
  clPointerColor = clBlack;
  SetPointerArt( pSLine );
  DefineSize();

  bitmap = new Graphics::TBitmap;

  // Timer einrichten und starten
  Timer = new TTimer(this);
  Timer->Interval = 1000;
  Timer->OnTimer = &OnTimer;
  SetClockSize( gSmall );
//  Timer->Enabled = true;
}
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
__fastcall TClock::~TClock()
{
  Timer->Enabled = false;
  delete bitmap;
}

//---------------------------------------------------------------------------
//      Paint
//---------------------------------------------------------------------------
void __fastcall TClock::Paint()
{
  Groesse gIndex = GetClockSize();
  if( gIndex == gSmall )
  {
        nCompoSize = 100;
        nZeigerFaktor = 1;
        Canvas->Font->Style = TFontStyles();
  }
  if( gIndex == gMiddle )
  {
        nCompoSize = 500;
        nZeigerFaktor = 2;
        Canvas->Font->Style = TFontStyles();
  }
  if( gIndex == gBig )
  {
        nCompoSize = 800;
        nZeigerFaktor = 3;
        Canvas->Font->Style = TFontStyles()<< fsBold;   // Fett
  }
  DefineSize();

  bitmap->Height = Height;
  bitmap->Width  = Width;
  ZeichneZiffern( *bitmap->Canvas );
  ZeichneAlleZeiger( *bitmap->Canvas );
  bitmap->Canvas->CopyRect( bmpRect , bitmap->Canvas, bmpRect );
} // Paint

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
void TClock::ZeichneZiffern( TCanvas &canvas )
{
    // Flle Hintergrund
    Canvas->Brush->Style = bsSolid;
    Canvas->Brush->Color = clBkUhrColor;
    Canvas->Rectangle( 0, 0, Width, Height );

    Canvas->Pen->Style = psSolid;
    Canvas->Pen->Color = clPointerColor;

//    Canvas->Brush->Color = PointerColor;
    Canvas->Font->Name = "MS Sans Serif";
    Canvas->Font->Size = nFontSize;
    Canvas->Font->Color = clPointerColor;

  // Setze Punkte fr die Stunden
  for( int n = 0; n < 360; n += 30 )
  {
    double x = ( sin( InRad( n ) ) * nRadiusUhr ) + nPosMitte;
    double y = ( cos( InRad( n ) ) * nRadiusUhr )*(-1) + nPosMitte;

    switch( n )
    {
      case 0:
//        Canvas->Brush->Style = psClear;
        Canvas->TextOut( x - ( nFontSize /1.5 ), y - ( nFontSize / 1.5 ), "12" );
        break;
      case 90:
//        Canvas->Brush->Style = psClear;
        Canvas->TextOut( x - (nFontSize / 3), y - ( nFontSize / 1.5 ), "3" );
        break;
      case 180:
//        Canvas->Brush->Style = psClear;
        Canvas->TextOut( x - ( nFontSize / 3 ), y - ( nFontSize  * 1.1), "6" );
        break;
      case 270:
//        Canvas->Brush->Style = psClear;
        Canvas->TextOut( x - (nFontSize / 3), y - ( nFontSize / 1.5 ), "9" );
        break;
      default:
//        Canvas->Brush->Style = psSolid;
        Canvas->Ellipse( x - nPunktRadius, y - nPunktRadius, x + nPunktRadius,
                    y + nPunktRadius );
      }
  }
} // ZeichneZiffern
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
void TClock::ZeichneAlleZeiger( TCanvas &canvas )
{
  Word Hour, Min, Sec, MSec;
  TDateTime dtPresent = Now();
  DecodeTime(dtPresent, Hour, Min, Sec, MSec);
  int nWinkel = 0;
  int nShowWidthPoint = 4 * nZeigerFaktor;
  
  // Stunden zeichnen
  nWinkel = (Hour * 30) + ((Min * 6) / 12 );
  ZeichneZeiger( *bitmap->Canvas, 2, nWinkel, nLaengeStdZeiger, Hour, nShowWidthPoint, FMyPointerArt );

  // Minuten zeichen
  nWinkel = Min * 6;
  ZeichneZeiger( *bitmap->Canvas, 2, nWinkel, nLaengeMinZeiger, Min, nShowWidthPoint, FMyPointerArt );

  // Sekunden zeichnen
  nWinkel = Sec * 6;
  ZeichneZeiger( *bitmap->Canvas, 1, nWinkel, nLaengeSecZeiger, Sec, nShowWidthPoint / 2 , FMyPointerArt );

} // ZeichneAlleZeiger
//---------------------------------------------------------------------------
//      ZeichneZeiger : Zeichnet jeweils nur einen Zeiger
//---------------------------------------------------------------------------
void TClock::ZeichneZeiger( TCanvas &canvas, int nLineWidth, int nWertWinkel,
                int nLaengeZeiger, int nPointerPos, int nPointWidth, bool bPointerArt )
{
  int nNewWinkel = 0;
  TPoint points[5];

  points[0].x = (sin( InRad( nWertWinkel )) * nLaengeZeiger ) + nPosMitte;
  points[0].y = ((cos( InRad( nWertWinkel )) * nLaengeZeiger )*(-1)) + nPosMitte;

  Canvas->Pen->Color = clPointerColor;
  Canvas->Pen->Style = psSolid;
  Canvas->Pen->Width = nLineWidth * nZeigerFaktor;

  if ( bPointerArt == 0 )
  {     // Zeiger als Line zeichnen
          Canvas->MoveTo( nPosMitte, nPosMitte );
          Canvas->LineTo( points[0].x, points[0].y );
  }
  else
  {     // Zeiger als Pfeil zeichnen
    nNewWinkel = nWertWinkel + 90;
    points[1].x = (sin( InRad( nNewWinkel )) * nPointWidth ) + nPosMitte;
    points[1].y = ((cos( InRad( nNewWinkel )) * nPointWidth )*(-1)) + nPosMitte;

    nNewWinkel = nWertWinkel + 180;
    points[2].x = (sin( InRad( nNewWinkel )) * nPointWidth ) + nPosMitte;
    points[2].y = ((cos( InRad( nNewWinkel )) * nPointWidth )*(-1)) + nPosMitte;

    nNewWinkel = nWertWinkel + 270;
    points[3].x = (sin( InRad( nNewWinkel )) * nPointWidth ) + nPosMitte;
    points[3].y = ((cos( InRad( nNewWinkel )) * nPointWidth )*(-1)) + nPosMitte;

    points[4].x = points[0].x;
    points[4].y = points[0].y;

    Canvas->Polyline(points,4);
  }
} // ZeichneZeiger
//---------------------------------------------------------------------------
//      OnTiner
//---------------------------------------------------------------------------
void __fastcall TClock::OnTimer( TObject* Sender )
{
  // In Entwurfsmodus Timer stoppen und zurckkehren
  if ( ComponentState.Contains( csDesigning ))
  {
    Timer->Enabled = false;
    return;
  }
  // Uhr zeichen
  bitmap->Height = Height;
  bitmap->Width  = Width;
  ZeichneZiffern( *bitmap->Canvas );
  ZeichneAlleZeiger( *bitmap->Canvas );
  bitmap->Canvas->CopyRect( bmpRect , bitmap->Canvas, bmpRect );
} // OnTimer
//---------------------------------------------------------------------------
//      SetClockSize -> Groesse der Uhr setzen
//---------------------------------------------------------------------------
void __fastcall TClock::SetClockSize( Groesse gWert )
{
        FMySize = gWert;
        Invalidate();
} // SetClockSize
//---------------------------------------------------------------------------
//      Aussehen der Zeiger ndern
//---------------------------------------------------------------------------
void __fastcall TClock::SetPointerArt( PointerStyle pSWert )
{
        FMyPointerArt = pSWert;
        Invalidate();
}  // SetPointerArt
//---------------------------------------------------------------------------
// Farbe vom Ziffernblatt ndern
//---------------------------------------------------------------------------
void __fastcall TClock::SetBkColor( TColor clBackground )
{
        clBkUhrColor = clBackground;
        Invalidate();
} // SetBkColor
//---------------------------------------------------------------------------
// Farbe der Zeiger ndern
//---------------------------------------------------------------------------
void __fastcall TClock::SetPointerColor( TColor clPColor )
{
        clPointerColor = clPColor;
        Invalidate();
} // SetPointerColor
//---------------------------------------------------------------------------
//      Variablen der Uhr setzen
//---------------------------------------------------------------------------
void TClock::DefineSize( void )
{
  nRand = 20;

  Width  = nCompoSize + nRand;
  Height = nCompoSize + nRand;
  nRadiusUhr = nCompoSize / 2;
  nPosMitte = ( (nCompoSize  + nRand ) / 2 );
  nPunktRadius = 2;
  nFontSize = 10;
  nLaengeSecZeiger = nRadiusUhr;
  nLaengeMinZeiger = nRadiusUhr;
  nLaengeStdZeiger = nRadiusUhr / 2;
  bmpRect = Rect( 0, 0, Width, Height );
} // DefineSize
//---------------------------------------------------------------------------
// Die Weite kann nur gelesen werden
//---------------------------------------------------------------------------
int __fastcall TClock::GetCompoWidth()
{
  if( GetClockSize() == gSmall ) return 120;
  if( GetClockSize() == gMiddle ) return 520;
  if( GetClockSize() == gBig ) return 820;

  return 0;
}  // GetCompoWidth
//---------------------------------------------------------------------------
// Die Hoehe kann nur gelesen werden
//---------------------------------------------------------------------------
int __fastcall TClock::GetCompoHeight()
{
  if( GetClockSize() == gSmall ) return 120;
  if( GetClockSize() == gMiddle ) return 520;
  if( GetClockSize() == gBig ) return 820;

  return 0;
} // GetCompoHeight
//---------------------------------------------------------------------------
//
namespace Clock
{
  void __fastcall PACKAGE Register()
  {
     TComponentClass classes[1] = {__classid(TClock)};
     RegisterComponents("BCB", classes, 0);
  }
}
//---------------------------------------------------------------------------
